@@ -1,27 +1,24 @@
+mod definitions_map;
 mod draft;
 mod error;
-mod r#type;
 mod schema;
 mod schematic;
 
+pub use definitions_map::DefinitionsMap;
 pub use draft::Draft;
 pub use error::Error;
-pub use r#type::{
-    AdditionalProperties,
-    ArrayKeys,
-    EnumKeys,
-    Items,
-    NumericKeys,
-    ObjectKeys,
-    Properties,
-    Property,
+pub use schema::{
+    r#type::*,
+    Definitions,
+    Schema,
     Type,
-    StringKeys,
-    TupleKeys,
 };
-pub use schema::Schema;
 pub use schematic::Schematic;
 
 /// Alias for a `Result` with the error type `rschema::Error`.
 /// 
 pub type Result<T> = std::result::Result<T, Error>;
+
+pub(crate) fn is_falsy(b: &Option<bool>) -> bool {
+    *b != Some(true)
+}
