@@ -3,14 +3,22 @@
 use seq_macro::seq;
 use paste::paste;
 
+use std::collections::{
+    HashMap,
+    HashSet,
+};
+
 use crate::{
     AdditionalProperties,
     ArrayKeys,
+    Definitions,
+    DefinitionsMap,
     EnumKeys,
     Items,
     NumericKeys,
     ObjectKeys,
     Properties,
+    Property,
     Type,
     StringKeys,
 };
@@ -31,7 +39,7 @@ use crate::{
 ///   - char
 ///   - str(&str)
 /// - **Compound types**:
-///   - [T; 0]
+///   - [T; N]
 ///   - tuples up to size 12
 /// - **Common standard library types**:
 ///   - String
@@ -39,6 +47,8 @@ use crate::{
 /// - **Wrapper types**:
 ///   - Box\<T\>
 /// - **Collection types**:
+///   - HashMap\<K, V, H\>
+///   - HashSet\<T, H\>
 ///   - Vec\<T\>
 /// 
 pub trait Schematic {
@@ -73,6 +83,94 @@ pub trait Schematic {
             None,
         )
     }
+
+    fn __defs() -> Definitions {
+        Self::__defs_map().build()
+    }
+
+    fn __defs_map() -> DefinitionsMap {
+        DefinitionsMap::new()
+    }
+}
+
+impl<T: Schematic> Schematic for &T {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+        unique_items: Option<bool>,
+    ) -> Type {
+        T::__type(
+            min_length,
+            max_length,
+            pattern,
+            format,
+            minimum,
+            maximum,
+            multiple_of,
+            exclusive_minimum,
+            exclusive_maximum,
+            min_items,
+            max_items,
+            unique_items,
+        )
+    }
+
+    fn __type_no_attr() -> Type {
+        T::__type_no_attr()
+    }
+
+    fn __defs_map() -> DefinitionsMap {
+        DefinitionsMap::new()
+    }
+}
+
+impl<T: Schematic> Schematic for &mut T {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+        unique_items: Option<bool>,
+    ) -> Type {
+        T::__type(
+            min_length,
+            max_length,
+            pattern,
+            format,
+            minimum,
+            maximum,
+            multiple_of,
+            exclusive_minimum,
+            exclusive_maximum,
+            min_items,
+            max_items,
+            unique_items,
+        )
+    }
+
+    fn __type_no_attr() -> Type {
+        T::__type_no_attr()
+    }
+
+    fn __defs_map() -> DefinitionsMap {
+        DefinitionsMap::new()
+    }
 }
 
 macro_rules! impl_for_str {
@@ -213,9 +311,10 @@ impl Schematic for () {
 }
 
 macro_rules! impls {
+    // $n: Member length
     // $t: Type parameter
     // $c: Comma
-    ( $( $t:tt $c:tt )* ) => {
+    ( $n:expr, $( $t:tt $c:tt )* ) => {
         impl<$($t:Schematic $c)*> Schematic for ($($t $c)*) {
             fn __type(
                 min_length: Option<u64>,
@@ -234,11 +333,17 @@ macro_rules! impls {
                 Type::Array(ArrayKeys {
                     items: Box::new(Items::Tuple(vec![
                         $(
-                            <$t as Schematic>::__type_no_attr(),
+                            Property {
+                                title: None,
+                                description: None,
+                                comment: None,
+                                deprecated: None,
+                                ty: <$t as Schematic>::__type_no_attr(),
+                            },
                         )*
                     ])),
-                    min_items,
-                    max_items,
+                    min_items: Some($n),
+                    max_items: Some($n),
                     unique_items,
                 })
             }
@@ -248,9 +353,9 @@ macro_rules! impls {
 
 macro_rules! impls_tuple {
     ($n:expr) => {
-        seq!(N in 0..$n {
+        seq!(N in 1..=$n {
             paste! {
-                impls!( #( [<T~N>], )* );
+                impls!( $n, #( [<T~N>], )* );
             }
         });
     };
@@ -268,6 +373,30 @@ macro_rules! impls_tuple_for {
 // See https://qiita.com/9laceef/items/e24f9541ef3924112f6b for these macros.
 impls_tuple_for!(12);
 
+impl<T: Schematic, const N: usize> Schematic for [T; N] {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+        unique_items: Option<bool>,
+    ) -> Type {
+        Type::Array(ArrayKeys {
+            items: Box::new(Items::Single(T::__type_no_attr())),
+            min_items: Some(N),
+            max_items: Some(N),
+            unique_items,
+        })
+    }
+}
+
 impl<T: Schematic> Schematic for Option<T> {
     fn __type(
         min_length: Option<u64>,
@@ -292,54 +421,6 @@ impl<T: Schematic> Schematic for Option<T> {
     }
 }
 
-impl<T: Schematic, const N: usize> Schematic for [T; N] {
-    fn __type(
-        min_length: Option<u64>,
-        max_length: Option<u64>,
-        pattern: Option<String>,
-        format: Option<String>,
-        minimum: Option<i64>,
-        maximum: Option<i64>,
-        multiple_of: Option<i64>,
-        exclusive_minimum: Option<i64>,
-        exclusive_maximum: Option<i64>,
-        min_items: Option<usize>,
-        max_items: Option<usize>,
-        unique_items: Option<bool>,
-    ) -> Type {
-        Type::Array(ArrayKeys {
-            items: Box::new(Items::Single(T::__type_no_attr())),
-            min_items: Some(N),
-            max_items: Some(N),
-            unique_items: unique_items,
-        })
-    }
-}
-
-impl<T: Schematic> Schematic for Vec<T> {
-    fn __type(
-        min_length: Option<u64>,
-        max_length: Option<u64>,
-        pattern: Option<String>,
-        format: Option<String>,
-        minimum: Option<i64>,
-        maximum: Option<i64>,
-        multiple_of: Option<i64>,
-        exclusive_minimum: Option<i64>,
-        exclusive_maximum: Option<i64>,
-        min_items: Option<usize>,
-        max_items: Option<usize>,
-        unique_items: Option<bool>,
-    ) -> Type {
-        Type::Array(ArrayKeys {
-            items: Box::new(Items::Single(T::__type_no_attr())),
-            min_items,
-            max_items,
-            unique_items,
-        })
-    }
-}
-
 impl<T: Schematic> Schematic for Box<T> {
     fn __type(
         min_length: Option<u64>,
@@ -372,7 +453,7 @@ impl<T: Schematic> Schematic for Box<T> {
     }
 }
 
-impl<V: Schematic, S> Schematic for std::collections::HashMap<String, V, S> {
+impl<V: Schematic, S> Schematic for HashMap<String, V, S> {
     fn __type(
         min_length: Option<u64>,
         max_length: Option<u64>,
@@ -396,3 +477,51 @@ impl<V: Schematic, S> Schematic for std::collections::HashMap<String, V, S> {
         })
     }
 }
+
+impl<T: Schematic, S> Schematic for HashSet<T, S> {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+        unique_items: Option<bool>,
+    ) -> Type {
+        Type::Array(ArrayKeys {
+            items: Box::new(Items::Single(T::__type_no_attr())),
+            min_items,
+            max_items,
+            unique_items: Some(true),
+        })
+    }
+}
+
+impl<T: Schematic> Schematic for Vec<T> {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+        unique_items: Option<bool>,
+    ) -> Type {
+        Type::Array(ArrayKeys {
+            items: Box::new(Items::Single(T::__type_no_attr())),
+            min_items,
+            max_items,
+            unique_items,
+        })
+    }
+}
